@@ -21,6 +21,8 @@ import re
 import string
 import subprocess
 import sys
+from datetime import datetime
+from textwrap import dedent
 
 try:
     from StringIO import StringIO
@@ -34,12 +36,16 @@ from pybuilder.core import (after,
                             task,
                             RequirementsFile,
                             Dependency)
-from pybuilder.errors import BuildFailedException
-from pybuilder.utils import as_list, is_string, is_notstr_iterable, get_dist_version_string
-from pybuilder.pip_utils import build_dependency_version_string
-from textwrap import dedent
-from pybuilder.pip_utils import pip_install
-from datetime import datetime
+from pybuilder.errors import BuildFailedException, MissingPrerequisiteException
+from pybuilder.utils import (as_list,
+                             is_string,
+                             is_notstr_iterable,
+                             get_dist_version_string,
+                             safe_log_file_name,
+                             assert_can_execute)
+# Plugin install_dependencies_plugin can reload pip_common and pip_utils. Do not use from ... import ...
+from pybuilder import pip_utils
+
 
 use_plugin("python.core")
 
@@ -66,16 +72,17 @@ $postinstall_script
 
 if __name__ == '__main__':
     setup(
-        name = '$name',
-        version = '$version',
-        description = '''$summary''',
-        long_description = '''$description''',
-        author = "$author",
-        author_email = "$author_email",
-        license = '$license',
-        url = '$url',
+        name = $name,
+        version = $version,
+        description = $summary,
+        long_description = $description,
+        author = $author,
+        author_email = $author_email,
+        license = $license,
+        url = $url,
         scripts = $scripts,
         packages = $packages,
+        namespace_packages = $namespace_packages,
         py_modules = $modules,
         classifiers = $classifiers,
         entry_points = $entry_points,
@@ -83,8 +90,11 @@ if __name__ == '__main__':
         package_data = $package_data,
         install_requires = $dependencies,
         dependency_links = $dependency_links,
-        zip_safe=True,
-        cmdclass={'install': install},
+        zip_safe = True,
+        cmdclass = {'install': install},
+        keywords = $setup_keywords,
+        python_requires = $python_requires,
+        obsoletes = $obsoletes,
     )
 """)
 
@@ -95,8 +105,14 @@ def default(value, default=""):
     return value
 
 
+def as_str(value):
+    return repr(str(value))
+
+
 @init
 def initialize_distutils_plugin(project):
+    project.plugin_depends_on("pypandoc", "~=1.3.0")
+
     project.set_property_if_unset("distutils_commands", ["sdist", "bdist_wheel"])
     project.set_property_if_unset("distutils_command_options", None)
 
@@ -112,6 +128,24 @@ def initialize_distutils_plugin(project):
     project.set_property_if_unset("distutils_upload_sign", False)
     project.set_property_if_unset("distutils_upload_sign_identity", None)
 
+    project.set_property_if_unset("distutils_readme_description", False)
+    project.set_property_if_unset("distutils_readme_file", "README.md")
+    project.set_property_if_unset("distutils_description_overwrite", False)
+
+    project.set_property_if_unset("distutils_console_scripts", None)
+    project.set_property_if_unset("distutils_entry_points", None)
+    project.set_property_if_unset("distutils_setup_keywords", None)
+
+
+@after("prepare")
+def set_description(project, logger):
+    if project.get_property("distutils_readme_description"):
+        try:
+            assert_can_execute(["pandoc", "--version"], "pandoc", "distutils")
+            doc_convert(project, logger)
+        except MissingPrerequisiteException:
+            logger.warn("Was unable to find pandoc and did not convert the documentation")
+
 
 @after("package")
 def write_setup_script(project, logger):
@@ -130,16 +164,17 @@ def render_setup_script(project):
 
     template_values = {
         "module": "setuptools" if project.get_property("distutils_use_setuptools") else "distutils.core",
-        "name": project.name,
-        "version": project.dist_version,
-        "summary": default(project.summary),
-        "description": default(project.description),
-        "author": author,
-        "author_email": author_email,
-        "license": default(project.license),
-        "url": default(project.url),
+        "name": as_str(project.name),
+        "version": as_str(project.dist_version),
+        "summary": as_str(default(project.summary)),
+        "description": as_str(default(project.description)),
+        "author": as_str(author),
+        "author_email": as_str(author_email),
+        "license": as_str(default(project.license)),
+        "url": as_str(default(project.url)),
         "scripts": build_scripts_string(project),
         "packages": build_packages_string(project),
+        "namespace_packages": build_namespace_packages_string(project),
         "modules": build_modules_string(project),
         "classifiers": build_classifiers_string(project),
         "entry_points": build_entry_points_string(project),
@@ -153,6 +188,9 @@ def render_setup_script(project):
             else ""),
         "preinstall_script": _normalize_setup_post_pre_script(project.setup_preinstall_script or "pass"),
         "postinstall_script": _normalize_setup_post_pre_script(project.setup_postinstall_script or "pass"),
+        "setup_keywords": build_setup_keywords(project),
+        "python_requires": as_str(default(project.requires_python)),
+        "obsoletes": build_string_from_array(project.obsoletes)
     }
 
     return SETUP_TEMPLATE.substitute(template_values)
@@ -206,10 +244,16 @@ def install_distribution(project, logger):
     _prepare_reports_dir(project)
     outfile_name = project.expand_path("$dir_reports", "distutils",
                                        "pip_install_%s" % datetime.utcnow().strftime("%Y%m%d%H%M%S"))
-    pip_install(project.expand_path("$dir_dist"), index_url=project.get_property("install_dependencies_index_url"),
-                extra_index_url=project.get_property("install_dependencies_extra_index_url"),
-                force_reinstall=True, logger=logger, verbose=project.get_property("verbose"), cwd=".",
-                outfile_name=outfile_name, error_file_name=outfile_name)
+    pip_utils.pip_install(
+        install_targets=project.expand_path("$dir_dist"),
+        index_url=project.get_property("install_dependencies_index_url"),
+        extra_index_url=project.get_property("install_dependencies_extra_index_url"),
+        force_reinstall=True,
+        logger=logger,
+        verbose=project.get_property("verbose"),
+        cwd=".",
+        outfile_name=outfile_name,
+        error_file_name=outfile_name)
 
 
 @task("upload", description="Upload a project to PyPi.")
@@ -227,6 +271,13 @@ def upload(project, logger):
         if sign_identity:
             upload_sign_args += ["--identity", sign_identity]
 
+    # Unfortunately, distutils/setuptools doesn't throw error if register fails
+    # but upload command will fail if project will not be registered
+    logger.info("Registering project %s-%s%s", project.name, project.version,
+                (" into repository '%s'" % repository) if repository else "")
+    register_cmd_line = [["register"] + repository_args]
+    execute_distutils(project, logger, register_cmd_line, False)
+
     logger.info("Uploading project %s-%s%s%s%s", project.name, project.version,
                 (" to repository '%s'" % repository) if repository else "",
                 get_dist_version_string(project, " as version %s"),
@@ -255,9 +306,9 @@ def execute_distutils(project, logger, distutils_commands, clean=False):
     for command in distutils_commands:
         logger.debug("Executing distutils command %s", command)
         if is_string(command):
-            output_file_path = os.path.join(reports_dir, command.replace("/", ""))
+            output_file_path = os.path.join(reports_dir, safe_log_file_name(command))
         else:
-            output_file_path = os.path.join(reports_dir, "__".join(command).replace("/", ""))
+            output_file_path = os.path.join(reports_dir, safe_log_file_name("__".join(command)))
         with open(output_file_path, "w") as output_file:
             commands = [sys.executable, setup_script]
             if project.get_property("verbose"):
@@ -295,7 +346,7 @@ def flatten_and_quote(requirements_file):
 
 
 def format_single_dependency(dependency):
-    return '%s%s' % (dependency.name, build_dependency_version_string(dependency))
+    return '%s%s' % (dependency.name, pip_utils.build_dependency_version_string(dependency))
 
 
 def build_install_dependencies_string(project):
@@ -410,6 +461,10 @@ def build_package_data_string(project):
     return result
 
 
+def build_namespace_packages_string(project):
+    return build_string_from_array([pkg for pkg in project.explicit_namespaces])
+
+
 def build_packages_string(project):
     return build_string_from_array([pkg for pkg in project.list_packages()])
 
@@ -446,6 +501,17 @@ def build_entry_points_string(project):
     return result
 
 
+def build_setup_keywords(project):
+    setup_keywords = project.get_property("distutils_setup_keywords")
+    if not setup_keywords or not len(setup_keywords):
+        return repr("")
+
+    if isinstance(setup_keywords, (list, tuple)):
+        return repr(" ".join(setup_keywords))
+
+    return repr(setup_keywords)
+
+
 def build_classifiers_string(project):
     classifiers = project.get_property('distutils_classifiers', [])
     return build_string_from_array(classifiers, indent=12)
@@ -503,6 +569,20 @@ def build_string_from_dict(d, indent=12):
     return result
 
 
+def doc_convert(project, logger):
+    import pypandoc
+    readme_file = project.expand_path("$distutils_readme_file")
+    logger.debug("Converting %s into RST format for PyPi documentation...", readme_file)
+    description = pypandoc.convert_file(readme_file, "rst")
+    if not hasattr(project, "description") or project.description is None or project.get_property(
+            "distutils_description_overwrite"):
+        setattr(project, "description", description)
+
+    if not hasattr(project, "summary") or project.summary is None or project.get_property(
+            "distutils_description_overwrite"):
+        setattr(project, "summary", description.splitlines()[0].strip())
+
+
 def _expand_leading_tabs(s, indent=4):
     def replace_tabs(match):
         return " " * (len(match.groups(0)) * indent)
